#include <avr/io.h>
#include <avr/interrupt.h>
#define F_CPU 16000000
#include <util/delay.h>

#define BASE 0.55 // Valoarea de baza a fact de umplere
#define SLOW1 (BASE / 1.90)
#define SLOW2 (BASE / 1.3)
#define SLOW3 (BASE / 1.30)
#define SLOW4 (BASE / 3.0)
#define SLOW5 (BASE / 4.0)
#define REV  (BASE / 1.4)
#define COMPB 1.09

void adc_init_bw(void){

  ADCSRA |= ((1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0));    //16Mhz/128 = 125Khz the ADC reference clock
  ADMUX |= (1<<REFS0);                //Voltage reference from Avcc (5v)
  ADCSRA |= (1<<ADEN);                //Turn on ADC
  ADCSRA |= (1<<ADSC);                //Do an initial conversion because this one is the slowest and to ensure that everything is up and running
}

int read_adc_bw(uint8_t channel){       //Function to read an arbitrary analogic channel/pin
  int adc_value, temp;

  /*
   * To do:
   * 1. Set ADC channel
   */
  ADMUX &= 0xE0;
  ADMUX |= channel;

  ADCSRA |= (1<<ADSC);                //Starts a new conversion
  while(!bit_is_set(ADCSRA,ADIF));    //Wait until the conversion is done: until ADIF from ADCSRA is set to 1
  temp = ADCL;
  adc_value = (ADCH<<8);
  adc_value |= temp;

  return adc_value;
}

void set_non_inverting_a() {
  /* set pins to 0 */
  TCCR1A &= ~((1 << COM1A1) | (1 << COM1A0));
  TCCR1A |= (1 << COM1A1);
  PORTD &= ~(1 << PD6);
}

void set_inverting_a() {
  /* set pins to 0 */
  TCCR1A &= ~((1 << COM1A1) | (1 << COM1A0));
  TCCR1A |= (1 << COM1A1) | (1 << COM1A0);
  PORTD |= (1 << PD6);
}

void set_non_inverting_b() {
  /* set pins to 0 */
  TCCR1A &= ~((1 << COM1B1) | (1 << COM1B0));
  TCCR1A |= (1 << COM1B1);
  PORTD &= ~(1 << PD3);
}

void set_inverting_b() {
  /* set pins to 0 */
  TCCR1A &= ~((1 << COM1B1) | (1 << COM1B0));
  TCCR1A |= (1 << COM1B1) | (1 << COM1B0);
  PORTD |= (1 << PD3);
}

void set_fill_factor_a(float factor) {
  uint16_t value = 1023 * factor;
  OCR1A = value;
}

void set_fill_factor_b(float factor) {
  uint16_t value = 1023 * factor * COMPB;
  OCR1B = value;
}

void enable_timer() {
  /* pinii setați pe output */
  DDRD |= (1 << PD4); //OC0A
  DDRD |= (1 << PD5); //OC0B

  TCCR1A &= 0x00; //SET all usefull bits to 0 first
  TCCR1B &= 0x00;

  /* Enable Fast PWM 10 bit for timer 1 */
  TCCR1A |= (1 << WGM10) | (1 << WGM11);
  TCCR0B |= (1 << WGM12);
  TCCR1B |= (1 << CS10); //No clock scaling

  set_non_inverting_a();
  set_non_inverting_b();
  set_fill_factor_a(BASE); //Left
  set_fill_factor_b(BASE); //Right

}


int main(){
  int i;

  DDRA = 0x00;
  DDRD = 0xFF;

  adc_init_bw();

  enable_timer();
  uint8_t sen;
  uint8_t prev = 0;

  while(1) {
    sen = 0;
    for (i = 0; i < 8; i++) {
      int value = read_adc_bw(i);
      if (value > 200) {
        sen |= (1 << i);
      }
    }

    if (sen == 0) {
      sen = prev;
    }
    set_non_inverting_a();
    set_non_inverting_b();

    if ((sen & (1 << 3)) && (sen & (1 << 4))) {
      set_fill_factor_a(BASE);
      set_fill_factor_b(BASE);
      PORTD |= (1 << PD7);
      goto end;
    } else {
      PORTD &= ~(1 << PD7);
    }

    if ((sen & (1 << 2)) && (sen & (1 << 3))) { //ma apropii de stânga
//            set_fill_factor_a(REV);
//      set_fill_factor_b(REV);
//      set_inverting_a();
set_fill_factor_a(SLOW1);
      set_fill_factor_b(BASE);
      goto end;
    }

    if ((sen & (1 << 1)) && (sen & (1 << 2))) { //ma apropii de stânga foarte mult
      set_fill_factor_a(SLOW3);
      set_fill_factor_b(BASE);
      goto end;
    }

    if ((sen & (1 << 0)) && (sen & (1 << 1))) { //ma apropii de stânga foarte mult
      set_fill_factor_a(SLOW5);
      set_fill_factor_b(BASE);
      goto end;
    }

    if (sen & (1 << 2)) {
//      set_fill_factor_a(REV);
//      set_fill_factor_b(REV);
//      set_inverting_a();
       set_fill_factor_b(BASE);
      set_fill_factor_a(SLOW2);
      goto end;
    }

    if (sen & (1 << 1)) {
      set_fill_factor_b(BASE);
      set_fill_factor_a(SLOW4);
      goto end;
    }

    if (sen & (1 << 0)) { // foarte aproape de a pierde linia

      set_fill_factor_a(REV);

      set_fill_factor_b(REV);
      set_inverting_a();
      goto end;
    }

    if ((sen & (1 << 4)) && (sen & (1 << 5))) { //ma apropii de stânga
//      set_fill_factor_a(REV);
//      set_fill_factor_b(REV);
//      set_inverting_b();
      set_fill_factor_b(SLOW1);
      set_fill_factor_a(BASE);
      PORTD |= (1 << PD7);
      goto end;
    } else {
      PORTD &= ~(1 << PD7);
    }

    if ((sen & (1 << 5)) && (sen & (1 << 6))) { //ma apropii de stânga foarte mult
 //     set_fill_factor_a(REV);
 //     set_fill_factor_b(REV);
 //     set_inverting_b();
      set_fill_factor_b(SLOW3);
      set_fill_factor_a(BASE);
      goto end;
    }

    if ((sen & (1 << 6)) && (sen & (1 << 7))) { //ma apropii de stânga foarte mult
      set_fill_factor_b(SLOW5);
      set_fill_factor_a(BASE);
      goto end;
    }

    if (sen & (1 << 5)) {
      set_fill_factor_a(REV);
      set_fill_factor_b(REV);
      set_inverting_b();
//set_fill_factor_a(BASE);
//      set_fill_factor_b(SLOW2);
      goto end;
    }

    if (sen & (1 << 6)) {
      set_fill_factor_a(BASE);
      set_fill_factor_b(SLOW4);
      goto end;
    }

    if (sen & (1 << 7)) { //foarte aproape de a pierde linia
      set_fill_factor_a(REV);
      set_fill_factor_b(REV);
      set_inverting_b();
      goto end;
    }

end: prev = sen;
  }

  return 0;
}
